import { Server as SocketIOServer, type Socket } from 'socket.io'
import 'dotenv/config'
import { SpotifyService } from '../spotify/service'
import { z } from 'zod'

const envSchema = z.object({
  CLIENT_ID: z.string().min(1),
  CLIENT_SECRET: z.string().min(1),
  REFRESH_TOKEN: z.string().min(1),
  CORS_ORIGIN: z.string().optional(),
})

const { CLIENT_ID, CLIENT_SECRET, REFRESH_TOKEN, CORS_ORIGIN } =
  envSchema.parse(process.env)

const io = new SocketIOServer(3000, {
  cors: {
    origin: CORS_ORIGIN ?? '*',
  },
})

const spotify = new SpotifyService({
  clientId: CLIENT_ID,
  clientSecret: CLIENT_SECRET,
  refreshToken: REFRESH_TOKEN,
})

let interval: ReturnType<typeof setInterval> | null = null
let currentSong: string | null = null

const sendNowPlayingData = async () => {
  const song = await spotify.getCurrentSong()
  try {
    switch (true) {
      case song?.is_playing && JSON.stringify(song) !== currentSong:
        currentSong = JSON.stringify(song)
        io.emit('nowPlayingData', currentSong)
        break
      case !song?.is_playing && currentSong !== null:
        if (currentSong == null) return
        currentSong = null
        io.emit('nowPlayingData', currentSong)
        break
      default:
        return
    }
  } catch (error) {
    console.error('Error fetching song data:', error)
  }
}

io.on('connection', async (socket: Socket) => {
  switch (currentSong) {
    case null:
      socket.emit('nowPlayingData', null)
      break
    default:
      socket.emit('nowPlayingData', currentSong)
      break
  }

  if (!interval) {
    interval = setInterval(() => {
      sendNowPlayingData().catch((err) => {
        console.error('Error sending NowPlayingData:', err)
      })
    }, 3000)
  }

  socket.on('disconnect', () => {
    if (io.sockets.sockets.size === 0 && interval) {
      clearInterval(interval)
      interval = null
    }
  })

  socket.on('error', (err) => {
    console.error('WebSocket error:', err)
  })
})

console.log(`\u001b[1;32m[SPOTIFY-WS] \x1b[34mSTARTED ON PORT 3000\x1b[0m\n`)
